#  Copyright 2020 Google LLC
#  Copyright 2020 EPAM Systems
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.


import os
import re
import sys

import pytest

sys.path.append(f"{os.getenv('AIRFLOW_SRC_DIR')}/plugins")

import mock_providers
from data import process_manifest_r2_op as test_data
from operators import process_manifest_r2


@pytest.mark.parametrize(
    "test_input, expected",
    [
        ("srn:type:work-product/WellLog:", "WellLog"),
        ("srn:type:file/las2:", "las2"),
    ]
)
def test_determine_data_type(test_input, expected):
   data_type = process_manifest_r2.determine_data_type(test_input)
   assert data_type == expected


@pytest.mark.parametrize(
    "data_type, loaded_conf, conf_payload, expected_file_result",
    [
        ("las2",
         test_data.LOADED_CONF,
         test_data.CONF_PAYLOAD,
         test_data.PROCESS_FILE_ITEMS_RESULT)
    ]
)
def test_process_file_items(data_type, loaded_conf, conf_payload, expected_file_result):
   file_id_regex = re.compile(r"srn\:file/" + data_type + r"\:\d+\:")
   expected_file_list = expected_file_result[0]
   file_list, file_ids = process_manifest_r2.process_file_items(loaded_conf, conf_payload)
   for i in file_ids:
      assert file_id_regex.match(i)

   for i in file_list:
      assert file_id_regex.match(i[0]["data"]["ResourceID"])
      i[0]["data"]["ResourceID"] = ""
   assert file_list == expected_file_list


@pytest.mark.parametrize(
    "dag_run",
    [
        (test_data.DAG_RUN_CONF)
    ]
)
def test_dataload_conf(dag_run):
    loaded_conf, conf_payload = process_manifest_r2.dataload(**dag_run)
    assert loaded_conf["acl"] == test_data.CONF["Payload"]["acl"]
    assert loaded_conf["legal_tag"] == test_data.CONF["Payload"]["legal"]
