#  Copyright 2021 Google LLC
#  Copyright 2021 EPAM Systems
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os
import sys
import pytest

sys.path.append(f"{os.getenv('AIRFLOW_SRC_DIR')}/plugins")
sys.path.append(f"{os.getenv('AIRFLOW_SRC_DIR')}/dags")

from libs.utils import split_id, remove_trailing_colon


class TestUtils:

    @pytest.mark.parametrize(
        "_id,expected_id_version",
        [
            pytest.param(
                "test:test:",
                ("test:test", ""),
                id="Trailing colon"),
            pytest.param(
                "test:test",
                ("test:test", ""),
                id="With no colon"),
            pytest.param(
                "test:test:1",
                ("test:test", "1"),
                id="With version")
        ]
    )
    def test_split_id_version(self, _id, expected_id_version):
        entity_id = split_id(_id)
        assert (entity_id.id, entity_id.version) == expected_id_version

    @pytest.mark.parametrize(
        "_id,expected_id",
        [
            pytest.param(
                "test:test:",
                "test:test",
                id="Trailing colon"),
            pytest.param(
                "test:test",
                "test:test",
                id="With no colon")
        ]
    )
    def test_delete_trailing_colon(self, _id, expected_id):
        assert remove_trailing_colon(_id) == expected_id
